
clear
grid_no  = input('mesh number = ');
dim      = grid_no^2;
% x        = linspace(0, 1, grid_no);
% y        = linspace(0, 1, grid_no);
leng_h   = 1 / ( grid_no + 1 );
x        = leng_h:leng_h:grid_no*leng_h;
y        = x;
[xx, yy] = meshgrid(x, y);
zz       = sin(pi*xx).*sin(pi*yy);

%tic;
mtx_A    = mtx_generate_2DLaplacian(grid_no);
rhs_f    = Construct_rhs(grid_no);
tic;
sol      = mtx_A \ rhs_f;
fprintf('CPU time for solving linear system = %14.4e, matrix size = %7.0f \n', toc, size(mtx_A,1));

tol   = eps * grid_no * 10;
maxit = 10^4;

mtx_U   = triu(mtx_A,1);
diag_D  = diag(mtx_A);
diag_Ds = diag_D.^(0.5);
mtx_D   = sparse(1:dim,1:dim,diag_D);
mtx_Ds  = sparse(1:dim,1:dim,diag_Ds);

tic;
[sol2,flag,relres,iter] = pcg(mtx_A, rhs_f, tol, maxit); 
fprintf('%3.0f iterations of CG with flag = %2.0f, RelRes = %11.4e and CPU time = %11.4e \n \n', ...
                           iter, flag, relres, toc);

for omega = 1:0.1:1.99;

    prec_M = mtx_D + omega * mtx_U;
    prec_M = mtx_Ds \ prec_M;

    tic;
    [sol3,flag,relres,iter] = pcg(mtx_A, rhs_f, 20*tol, maxit, prec_M', prec_M); 
    fprintf('%3.0f iterations of CG with flag = %2.0f, RelRes = %11.4e and CPU time = %11.4e \n', ...
                           iter, flag, relres, toc);
end
                       
mtx_sol  = zeros(grid_no);
for ii = 1:grid_no
    mtx_sol(ii,:) = sol((ii-1)*grid_no+1:ii*grid_no);
end

figure(1)
mesh(xx, yy, zz);
xlabel('x');
ylabel('y');
zlabel('u');
title('u(x,y) = sin( x \pi ) sin( y \pi )');

figure(2)
surf(xx, yy, zz);
shading interp;
xlabel('x');
ylabel('y');
zlabel('u');
title('u(x,y) = sin( x \pi ) sin( y \pi )');

figure(3)
surf(xx, yy, mtx_sol);
shading interp;
xlabel('x');
ylabel('y');
zlabel('u');
title('Approximate solution');

figure(4)
surf(xx, yy, abs(mtx_sol-zz));
% shading interp;
xlabel('x');
ylabel('y');
zlabel('u');
title('Absolute Error');
